// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.IO;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// This class represents a configuration file and stores internally
    /// all the information needed to write back the configuration file
    /// if needed. The file can be written in the same format of the
    /// original or in any other supported format.
    /// </summary>
    public class ConfigFile : MarshalByRefObject
    {
        /// <summary>
        /// The currently implemented configuration file handlers are
        /// listed here. When an additional handler is implemented it
        /// should be added here.
        /// </summary>
        private static readonly FileHandler[] fileHandlers =
        {
            new IniFileHandler(),
            new XmlFileHandler(),
            new PluggableHandler() //added by TCSDEVELOPER for version 2.0
        };
        
        /// <summary>
        /// The name of the original file.
        /// </summary>
        private string filename = null;
        
        /// <summary>
        /// The file handler used to process the original file.
        /// </summary>
        private FileHandler fileHandler = null;
        
        /// <summary>
        /// The contained namespaces.
        /// </summary>
        private Namespace[] namespaces = null;

        /// <summary>
        /// The new namespace (needed for Reload).
        /// </summary>
        private string newNamespace = null;
        
        /// <summary>
        /// The error callback (needed for Reload).
        /// </summary>
        private ErrorCallback errorCallback = null;

        /// <summary>
        /// Internal constructor.
        /// It tries to find an appropriate handler for the file by
        /// iterating through a list of implemented handlers and then 
        /// loads the file using the handler.
        /// </summary>
        /// 
        /// <param name="filename">The filename to process.</param>
        /// <param name="newNamespace">
        /// The new namespace to load the properties in or null in which
        /// case it is ignored.
        /// This means that all namespaces from the config file will be
        /// ignored and all properties will go into the new package.
        /// </param>
        /// <param name="errorCallback">The error callback.</param>
        internal ConfigFile(string filename, string newNamespace, 
                            ErrorCallback errorCallback)
        {
            this.filename = filename;
            // Find the correct file handler
            fileHandler = null;
            foreach (FileHandler fileHandlerObject in fileHandlers)
            {
                if (fileHandlerObject.Supports(filename))
                {
                    fileHandler = fileHandlerObject;
                    break;
                }
            }
            if (fileHandler == null)
            {
                throw new UnknownFormatException();
            }
            namespaces = fileHandler.LoadFromFile(filename, errorCallback);
            if (newNamespace != null)
            {
                Namespace newNamespaceObject = new Namespace(newNamespace);
                foreach (Namespace theNamespace in namespaces)
                {
                    foreach (Property prop in theNamespace.Properties)
                    {
                        newNamespaceObject.AddProperty(prop, errorCallback);
                    }
                }
                namespaces = new Namespace[1];
                namespaces[0] = newNamespaceObject;
                this.newNamespace = newNamespace;
            }
            this.errorCallback = errorCallback;
        }
                
        /// <summary>
        /// The Namespaces value.
        /// </summary>
        /// <value>
        /// The list of Namespace objects in the configuration file.
        /// </value>
        public Namespace[] Namespaces
        {
            get
            {
                return (Namespace[])namespaces.Clone();
            }
        }
        
        /// <summary>
        /// Reloads the configuration file.
        /// </summary>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be saved because of that.
        /// </exception>
        internal void Reload()
        {
            foreach (Namespace theNamespace in namespaces)
            {
                if (theNamespace.isMerged)
                {
                    throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                }
                foreach (Property prop in theNamespace.Properties)
                {
                    if (prop.isMerged)
                    {
                        throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                    }
                }
            }
            namespaces = fileHandler.LoadFromFile(filename, errorCallback);
            if (newNamespace != null)
            {
                Namespace newNamespaceObject = new Namespace(newNamespace);
                foreach (Namespace theNamespace in namespaces)
                {
                    foreach (Property prop in theNamespace.Properties)
                    {
                        newNamespaceObject.AddProperty(prop, errorCallback);
                    }
                }
                namespaces = new Namespace[1];
                namespaces[0] = newNamespaceObject;
            }
        }
                
        /// <summary>
        /// Save the changes back to disk in the same format
        /// as the original.
        /// </summary>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be saved because of that.
        /// </exception>
        internal void Save()
        {
            foreach (Namespace theNamespace in namespaces)
            {
                if (theNamespace.isMerged)
                {
                    throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                }
                foreach (Property prop in theNamespace.Properties)
                {
                    if (prop.isMerged)
                    {
                        throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                    }
                }
            }
            fileHandler.SaveFile(namespaces, filename);
        }
        
        /// <summary>
        /// Save the changes back to disk in a different file from the
        /// original (the format may differ as it is selected based
        /// on the extension).
        /// </summary>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be saved because of that.
        /// </exception>
        internal void SaveTo(string filename)
        {
            foreach (Namespace theNamespace in namespaces)
            {
                if (theNamespace.isMerged)
                {
                    throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                }
                foreach (Property prop in theNamespace.Properties)
                {
                    if (prop.isMerged)
                    {
                        throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                    }
                }
            }
            // Find the correct file handler
            foreach (FileHandler fh in fileHandlers)
            {
                if (fh.Supports(filename))
                {
                    fh.SaveFile(namespaces, filename);
                    break;
                }
            }
        }                
    }
}
