// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER
// @version 2.0

using System;
using System.Collections;
using System.IO;
using System.Configuration;
using System.Reflection;

namespace TopCoder.Util.ConfigurationManager
{    
    /// <summary>
    /// This is the main class which provides the interaction with the
    /// user (a singleton). The class exposes methods for loading new
    /// configuration files and getting information about loaded namespaces,
    /// properties and their values.
    /// </summary>
    
    
//     Version 2.0: Removed the System Wide Singleton and its associated methods.
//     Added the following methods:
//     
//     public object CreateInstance (string ns, string typeProperty, Type baseType)
//     public object CreateInstance (string ns, string typeProperty, object[] args, Type baseType)
//     public object CreateInstance (string ns, string typeProperty, string assemblyProperty, 
//     Type baseType)
//     public object CreateInstance (string ns, string typeProperty, string assemblyProperty,
//     object[] args, Type baseType)
//     public string GetValue (string ns, string property, IParsingPreference preference)
//     public string[] GetValues (string ns, string property, IParsingPreference preference)
//     public object GetValue (string ns, string property, Type type)
//     public Array GetValues (string ns, string property, Type type)
//     public object GetValue (string ns, string property, Type type, IStringConverter converter)       
//     public Array GetValues (string ns, string property, Type type, IStringConverter converter)
//     public int GetIntValue (string ns, string property)
//     public double GetDoubleValue (string ns, string property)
//     public int[] GetIntValues (string ns, string property)
//     public double[] GetDoubleValues (string ns, string property)
 

    public class ConfigManager: MarshalByRefObject
    {
        /// <summary>
        /// The name of the file with the names of the configuration files to
        /// be preloaded.
        /// </summary>
        private static string preloadFile = @"..\..\conf\preload.xml";
        
        /// <summary>
        /// The one instance of the class.
        /// </summary>
        private static ConfigManager instance = null;

        /// <summary>
        /// Maps namespace names to Namespace objects.
        /// </summary>
        private Hashtable namespaceLookup = Hashtable.Synchronized(new Hashtable());
        
        /// <summary>
        /// Maps config file names to ConfigFile objects.
        /// </summary>
        private Hashtable configFileLookup = Hashtable.Synchronized(new Hashtable());
        
        /// <summary>
        /// Initializes the preload file location from the application configuration file, using the
        /// key "TopCoder.Util.ConfigurationManager.Preload".  If this setting is not found, the default
        /// value is "..\..\conf\preload.xml".
        /// </summary>
        static ConfigManager()
        {
            if (ConfigurationSettings.AppSettings["TopCoder.Util.ConfigurationManager.Preload"] != null)
            {
                preloadFile = ConfigurationSettings.AppSettings["TopCoder.Util.ConfigurationManager.Preload"];
            }
        }

        /// <summary>
        /// Private constructor. Prevents instantiation from outside the class.
        /// Its task is to load the preloadFile and extract from there all
        /// the names of the configuration files that need to be preloaded.
        /// Then each file will be loaded.
        /// </summary>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the argument is null.
        /// </exception>
        private ConfigManager()
        {
            ConfigFile configFile = new ConfigFile(preloadFile, null, null);
            if (configFile.Namespaces.Length != 0)
            {
                string[] filenames = configFile.Namespaces[0].GetProperty("preload").Values;
                foreach (string filename in filenames)
                {
                    LoadFile(filename);
                }
            }
        }
        
        /// <summary>
        /// Get the instance of the configuration manager
        /// (see the spec on remoting).
        /// </summary>
        /// 
        /// <returns>The instance</returns>
        public static ConfigManager GetInstance()
        {
            lock(typeof(ConfigManager))
            {
                if (instance == null)
                {
                    instance = new ConfigManager();
                }
                return instance;
            }
        }
        

        /// <summary>
        /// Loads a new configuration file.
        /// All exceptions will be thrown.
        /// </summary>
        /// 
        /// <param name="filename">The file to load from.</param>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the filename argument is null.
        /// </exception>
        public ConfigFile LoadFile(string filename)
        {
            return LoadFile(filename, null, null);
        }
        
        /// <summary>
        /// Loads a new configuration file and forces all properties to
        /// be placed in the specified namespace (their namespace will
        /// be striped). All exceptions will be thrown.
        /// </summary>
        /// 
        /// <param name="filename">The file to load from.</param>
        /// <param name="newNamespace">The new namespace.</param>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the filename argument is null.
        /// </exception>
        public ConfigFile LoadFile(string filename, string newNamespace)
        {
            return LoadFile(filename, newNamespace, null);
        }
        
        /// <summary>
        /// Loads a new configuration file.
        /// The exceptions will be thrown according to the callback
        /// given by the user.
        /// </summary>
        /// 
        /// <param name="filename">The file to load from.</param>
        /// <param name="errorCallback">The error callback.</param>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the filename argument is null.
        /// </exception>
        public ConfigFile LoadFile(string filename, 
            ErrorCallback errorCallback)
        {
            return LoadFile(filename, null, errorCallback);
        }
        
        /// <summary>
        /// Loads a new configuration file and forces all properties to
        /// be placed in the specified namespace (their namespace will
        /// be striped). The exceptions will be thrown according to the
        /// callback given by the user.
        /// </summary>
        /// 
        /// <param name="filename">The file to load from.</param>
        /// <param name="newNamespace">The new namespace.</param>
        /// <param name="errorCallback">The error callback.</param>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the filename argument is null.
        /// </exception>
        public ConfigFile LoadFile(string filename, string newNamespace, 
            ErrorCallback errorCallback)
        {
            if (filename == null)
            {
                throw new NullReferenceException();
            }
            if (!configFileLookup.ContainsKey(filename))
            {
                ConfigFile configFile = new ConfigFile(filename, newNamespace, errorCallback);
                configFileLookup.Add(filename, configFile);
                InsertNewNamespaces(configFile.Namespaces, errorCallback);
                return configFile;
            }
            else
            {
                throw new NamespaceClashException(filename + " has already been loaded.");
            }
        }
        
        /// <summary>
        /// This helper method adds new namespaces to the lookup table and
        /// deals with clashes
        /// </summary>
        /// <param name="namespaces">The new namespaces to be added</param>
        /// <param name="errorCallback">The error callback</param>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        ///
        /// <exception cref="NullReferenceException">
        /// If the argument is null.
        /// </exception>
        private void InsertNewNamespaces(Namespace[] namespaces, ErrorCallback errorCallback)
        {
            if (namespaces == null)
            {
                throw new NullReferenceException();
            }
            for (int i = 0; i < namespaces.Length; i++)
            {
                Namespace theNamespace = (Namespace)namespaces[i];
                if (namespaceLookup[theNamespace.Name] == null)
                {
                    lock (this)
                    {
                        namespaceLookup.Add(theNamespace.Name, theNamespace);
                    }
                }
                else
                {
                    // A clash occured either within the new namespaces being
                    // inserted or with the original values.  Namespaces that
                    // are higher up are considered to be the "old" values.
                    if (errorCallback != null)
                    {
                        lock (this)
                        {
                            ErrorAction errorAction = errorCallback.NamespaceCollision((Namespace)namespaceLookup[theNamespace.Name], theNamespace);
                            if (errorAction == ErrorAction.ReplaceOld)
                            {
                                namespaceLookup.Remove(theNamespace.Name);
                                namespaceLookup.Add(theNamespace.Name, theNamespace);
                            }
                            else if (errorAction == ErrorAction.KeepOld)
                            {
                                // Do nothing, the old one is already in place
                            }
                            else if (errorAction == ErrorAction.Merge)
                            {
                                Namespace old = (Namespace)namespaceLookup[theNamespace.Name];
                                foreach (Property prop in theNamespace.Properties)
                                {
                                    old.AddProperty(prop, errorCallback);
                                }
                                old.isMerged = true;
                                theNamespace.isMerged = true;
                            }
                            else if (errorAction == ErrorAction.ThrowException)
                            {
                                throw new NamespaceClashException("Namespace clash occurred and ErrorCallback specified to throw exception");
                            }
                        }
                    }
                    else
                    {
                        throw new NamespaceClashException(theNamespace.Name + " is already present");
                    }
                }
            }
        }

        /// <summary>
        /// Removes the config file and the associated namespaces from the lookup tables.
        /// </summary>
        /// 
        /// <param name="filename">The config filename.</param>
        /// <returns>
        /// true if the config file was found and removed, false otherwise.
        /// </returns>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be removed because of that.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the argument is null.
        /// </exception>
        public bool RemoveFile(string filename)
        {
            lock (this)
            {
                ConfigFile configFile = GetConfigFile(filename);
                if (configFile != null)
                {
                    foreach (Namespace theNamespace in configFile.Namespaces)
                    {
                        if (theNamespace.isMerged)
                        {
                            throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                        }
                        foreach (Property prop in theNamespace.Properties)
                        {
                            if (prop.isMerged)
                            {
                                throw new MergeException("The namespaces or properties have been merged and the file is not safe to save.");
                            }
                        }
                    }
                    foreach (Namespace theNamespace in configFile.Namespaces)
                    {
                        namespaceLookup.Remove(theNamespace.Name);
                    }
                    configFileLookup.Remove(filename);
                }
                return (configFile != null);
            }
        }

        /// <summary>
        /// Reloads a config file by removing its namespaces and then reloading it using
        /// the ConfigFile.Reload method.
        /// </summary>
        /// 
        /// <param name="filename">The config filename.</param>
        /// <returns>
        /// true if the config file was found and reloaded, false otherwise.
        /// </returns>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be unloaded because of that.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the argument is null.
        /// </exception>
        public bool ReloadFile(string filename) 
        {
            lock (this)
            {
                ConfigFile configFile = GetConfigFile(filename);
                if (configFile != null)
                {
                    foreach (Namespace theNamespace in configFile.Namespaces)
                    {
                        namespaceLookup.Remove(theNamespace.Name);
                    }
                    configFile.Reload();
                }
                else
                {
                    configFile = new ConfigFile(filename, null, null);
                    configFileLookup.Add(filename, configFile);
                }
                foreach (Namespace theNamespace in configFile.Namespaces)
                {
                    namespaceLookup.Add(theNamespace.Name, theNamespace);
                }
                // Technically this will always be true because
                // if we can't find the file already loaded, we
                // just go ahead and load it normally.
                return true;
            }
        }
        
        /// <summary>
        /// Save a config file back to disk in the same format as the original.
        /// </summary>
        /// 
        /// <param name="filename">The config filename.</param>
        /// <returns>
        /// true if the config file was found and saved, false otherwise.
        /// </returns>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be saved because of that.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If the argument is null.
        /// </exception>
        public bool SaveFile(string filename)
        {
            lock (this)
            {
                ConfigFile configFile = GetConfigFile(filename);
                if (configFile != null)
                {
                    configFile.Save();
                }
                return (configFile != null);
            }
        }
        
        /// <summary>
        /// Save a config file back to disk in a different file from the
        /// original (the format may differ as it is selected based
        /// on the extension).
        /// </summary>
        /// 
        /// <param name="filename">The config filename.</param>
        /// <param name="newFilename">The filename to save to.</param>
        /// <returns>
        /// true if the config file was found and saved, false otherwise.
        /// </returns>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        /// 
        /// <exception cref="MergeException">
        /// If any of the namespaces or properties of the config file are merged
        /// and cannot be saved because of that.
        /// </exception>
        /// 
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        public bool SaveFileTo(string filename, string newFilename)
        {
            lock (this)
            {
                ConfigFile configFile = GetConfigFile(filename);
                if (configFile != null)
                {
                    configFile.SaveTo(newFilename);
                }
                return (configFile != null);
            }
        }

        /// <summary>
        /// The ConfigFiles property.
        /// </summary>
        /// <value>
        /// The array of config files which are currently loaded.
        /// The results are in no particular order.
        /// </value>
        public ConfigFile[] ConfigFiles
        {
            get
            {
                ConfigFile[] files = new ConfigFile[configFileLookup.Count];
                configFileLookup.Values.CopyTo(files, 0);
                return files;
            }
        }
        
        /// <summary>
        /// Get a config file by its filename.
        /// </summary>
        /// 
        /// <param name="filename">The config filename.</param>
        /// <returns>
        /// The config file or null if not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If argument is null.
        /// </exception>
        public ConfigFile GetConfigFile(string filename)
        {
            if (filename == null)
            {
                throw new NullReferenceException();
            }
            return (ConfigFile)configFileLookup[filename];
        }
        
        /// <summary>
        /// The Namespaces property.
        /// </summary>
        /// <value>
        /// The array of namespaces which are currently loaded.
        /// The results are in no particular order.
        /// </value>
        public Namespace[] Namespaces
        {
            get
            {
                lock (this)
                {
                    Namespace[] namespaces = new Namespace[namespaceLookup.Count];
                    namespaceLookup.Values.CopyTo(namespaces, 0);
                    return namespaces;
                }
            }
        }
        
        /// <summary>
        /// Get a namespace by its name.
        /// </summary>
        /// 
        /// <param name="name">The namespace.</param>
        /// <returns>
        /// The namespace or null if not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If argument is null.
        /// </exception>
        public Namespace GetNamespace(string name)
        {
            if (name == null)
            {
                throw new NullReferenceException();
            }
            return (Namespace)namespaceLookup[name];
        }
        
        /// <summary>
        /// Shortcut for getting a namespace by its name.
        /// Same as GetNamespace(string name).
        /// </summary>
        /// 
        /// <param name="name">The namespace.</param>
        /// <returns>
        /// The namespace or null if not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If argument is null.
        /// </exception>
        public Namespace this[string name]
        {
            get
            {
                return GetNamespace(name);
            }
        }
        
        /// <summary>
        /// Shortcut for GetNamespace(namespaceString).GetProperty(property).
        /// Get a property by name.
        /// </summary>
        /// 
        /// <param name="namespaceString">The namespace to look in.</param>
        /// <param name="property">The name of the property.</param>
        /// <returns>
        /// The property or null if not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        public Property GetProperty(string namespaceString, string property)
        {
            return GetNamespace(namespaceString).GetProperty(property);
        }
        
        /// <summary>
        /// Shortcut for GetNamespace(namespaceString).GetProperty(property).Value.
        /// Get the value of a property.
        /// </summary>
        /// 
        /// <param name="namespaceString">The namespace to look in.</param>
        /// <param name="property">The name of the property.</param>
        /// <returns>
        /// The value of the property (the first value if there are more)
        /// or null if not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        public string GetValue(string namespaceString, string property)
        {
            if (namespaceString == null || property == null)
            {
                throw new NullReferenceException();
            }
            Namespace theNamespace = GetNamespace(namespaceString);
            if (theNamespace == null)
            {
                return null;
            }
            Property theProperty = theNamespace.GetProperty(property);
            if (theProperty == null)
            {
                return null;
            }
            return theProperty.Value;
        }
        
        /// <summary>
        /// Shortcut for GetNamespace(namespaceString).GetProperty(property).Values.
        /// Get the values of a property.
        /// </summary>
        /// 
        /// <param name="namespaceString">The namespace to look in.</param>
        /// <param name="property">The name of the property.</param>
        /// <returns>
        /// The values of the property or null if not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        public string[] GetValues(string namespaceString, string property)
        {
            if (namespaceString == null || property == null)
            {
                throw new NullReferenceException();
            }
            Namespace theNamespace = GetNamespace(namespaceString);
            if (theNamespace == null)
            {
                return null;
            }
            Property theProperty = theNamespace.GetProperty(property);
            if (theProperty == null)
            {
                return null;
            }
            return theProperty.Values;
        }
        
        /// <summary>
        /// Shortcut for GetNamespace(namespaceString).GetProperty(property).Value.
        /// Set the value of a property.
        /// </summary>
        /// 
        /// <param name="namespaceString">The namespace to look in.</param>
        /// <param name="property">The name of the property.</param>
        /// <param name="value">The new value.</param>
        /// <returns>
        /// true if the property was found and its value was updated,
        /// false if the property was not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        public bool SetValue(string namespaceString, string property, string value)
        {
            if (namespaceString == null || property == null || value == null)
            {
                throw new NullReferenceException();
            }
            Namespace theNamespace = GetNamespace(namespaceString);
            if (theNamespace != null)
            {
                Property theProperty = theNamespace.GetProperty(property);
                if (theProperty != null)
                {
                    theProperty.Value = value;
                    return true;
                }
            }
            return false;
        }
        
        /// <summary>
        /// Shortcut for GetNamespace(namespaceString).GetProperty(property).Values.
        /// Set the values of a property.
        /// </summary>
        /// 
        /// <param name="namespaceString">The namespace to look in.</param>
        /// <param name="property">The name of the property.</param>
        /// <param name="values">The new values.</param>
        /// <returns>
        /// true if the property was found and its values were updated,
        /// false if the property was not found.
        /// </returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// If any argument is null or if the elements of the array are null.
        /// </exception>
        public bool SetValues(string namespaceString, string property, string[] values)
        {
            if (namespaceString == null || property == null || values == null)
            {
                throw new NullReferenceException();
            }
            Namespace theNamespace = GetNamespace(namespaceString);
            if (theNamespace != null)
            {
                Property theProperty = theNamespace.GetProperty(property);
                if (theProperty != null)
                {
                    theProperty.Values = values;
                    return true;
                }
            }
            return false;
        }
        
        /// <summary>
        /// Refreshes the config manager by reloading each config file.
        /// </summary>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public void Refresh()
        {
            lock (this)
            {
                namespaceLookup.Clear();
            }
            foreach (string filename in configFileLookup.Keys)
            {
                ReloadFile(filename);
            }
        }

        /// <summary>
        /// Clears the config manager of all files and namespaces
        /// </summary>
        public void Clear()
        {
            Clear(true);
        }

        /// <summary>
        /// Clear the config manager of all files and namespaces and reloads
        /// the preload files if required.
        /// </summary>
        /// <param name="preload">Reload the preload files?</param>
        public void Clear(bool preload)
        {
            lock(this)
            {
                namespaceLookup.Clear();
                configFileLookup.Clear();
                if (preload)
                {
                    ConfigFile configFile = new ConfigFile(preloadFile, null, null);
                    if (configFile.Namespaces.Length != 0)
                    {
                        string[] filenames = configFile.Namespaces[0].GetProperty("preload").Values;
                        foreach (string filename in filenames)
                        {
                            LoadFile(filename);
                        }
                    }
                }
            }
        }

        // Copyright (c)2005, TopCoder, Inc. All rights reserved
        // @author TCSDEVELOPER
        // @version 2.0
        // The code below contains enhancements for version 2.0

        /// <summary>
        /// Private function to create an instance of a type from the given information.
        /// </summary>
        /// <param name="ns">Namespace where the property and type are located.</param>
        /// <param name="typeProperty">Property name that holds the typename.</param>
        /// <param name="assemblyProperty">Property to get the assembly from which to create the requested object.</param>
        /// <param name="args">Arguments to be passed to the constructor of the object to be created.</param>
        /// <param name="baseType">Base typename determines if typeProperty is a valid typename.</param>
        /// <returns>The created object or null if unable to create.</returns>
        /// <exception cref="InstanceCreationException">
        /// If the instance cannot be created or the assembly cannot be loaded.
        /// </exception>

        private object CreateObject (string ns, string typeProperty, string assemblyProperty,
            object[] args, Type baseType)
        {
            // It will call GetValue method to get the type name and assembly name first; 
            // if the assemblyProperty is null, it will be ignored. Then create the 
            // Type instance from the retrieved assembly and typename with 
            // Assembly.LoadFrom(assemblyName).GetType(typeName) or Type.GetType(typeName) 
            // depending on the assemblyName existing or not. 
            // Then check the created Type with Type.IsSubclassOf(baseType) to make 
            // sure it extends from the baseType. 
            // Finally using Activator.CreateInstance(type, args) to create the instance.

            
            object inst = null;
            Type t = null;

            string typeName = GetValue (ns, typeProperty);

            if (typeName == null)
            {
                throw new InstanceCreationException (typeProperty + " cannot be found in namespace " + ns);
            }

            try
            {
                // if assembly property is mentioned, get the assembly name
                // and get the type from the assembly.
                if (assemblyProperty != null)
                {
                    string assemblyName = GetValue (ns, assemblyProperty);
                    Assembly a = Assembly.LoadFrom(assemblyName);
                    t = a.GetType(typeName, true, true);
                }
                else
                {
                    t = Type.GetType(typeName, true, true);
                }

            }
            catch (Exception e)
            {
                throw new InstanceCreationException(e.Message);
            }

            if (baseType != null && !baseType.IsAssignableFrom(t))
            {
                throw new InstanceCreationException(typeName + 
                    " is not a derived class of " + baseType);
            }

            if (args != null)
            {
                inst = Activator.CreateInstance(t, args);
            }
            else
            {
                inst  = Activator.CreateInstance(t);
            }

            return inst;
        }


        /// <summary>
        /// Instantiates an object based on the typename found in the 
        /// configuration property of a specific namespace. 
        /// </summary>
        /// <param name="ns">Namespace where the typename is located.</param>
        /// <param name="typeProperty">The property that holds the typename.</param>
        /// <param name="baseType">
        /// Base typename determines if typeProperty is a valid typename.
        /// </param>
        /// <returns>The created object or null if unable to create.</returns>
        /// <exception cref="InstanceCreationException">
        /// If the instance cannot be created or the assembly cannot be loaded.
        /// </exception>
        /// <exception cref="ArgumentNullException">When any argument is null.</exception>
        /// <exception cref="ArgumentException">When ns or typeProperty is empty.</exception>
       
        public object CreateInstance (string ns, string typeProperty, Type baseType)
        {
            //  A required base type of interface is  specified 
            // in order to enforce this type dependency when determing if the requested 
            // property is valid . This method will create the instance without the 
            // assembly name, and with the no-arg constructor. It will simply delegate to 
            // CreateObject(ns, typeProperty, null, baseType).

            if (ns == null || typeProperty == null || baseType == null)
            {
                throw new ArgumentNullException ();
            }

            if (ns.Trim().Length == 0 || typeProperty.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            return CreateObject (ns, typeProperty, null, null, baseType);
        }

        /// <summary>
        /// Instantiates an object based on the typename found in the 
        /// configuration property of a specific namespace. 
        /// </summary>
        /// <param name="ns">Namespace where the property and type are located.</param>
        /// <param name="typeProperty">Property name that holds the typename.</param>
        /// <param name="args">
        /// Arguments to be passed to the constructor of the object to be created.
        /// </param>
        /// <param name="baseType"> 
        /// Base typename determines if typeProperty is a valid typename.
        /// </param>
        /// <returns>The created object or null if unable to create.</returns>
        /// <exception cref="InstanceCreationException">
        /// If the instance cannot be created or the assembly cannot be loaded.
        /// </exception>
        /// <exception cref="ArgumentNullException">When any argument is null.</exception>
        /// <exception cref="ArgumentException">When ns or typeProperty is empty.</exception>
       
        public object CreateInstance (string ns, string typeProperty, object[] args, 
            Type baseType)
        {
            // This method will create the instance without the assembly name, 
            // and with constructor taking arguments. It will simply delegate to 
            // CreateObject(ns, typeProperty, null, args, baseType).

            if (ns == null || typeProperty == null || args == null || baseType == null)
            {
                throw new ArgumentNullException ();
            }

            if (ns.Trim().Length == 0 || typeProperty.Trim().Length == 0 )
            {
                throw new ArgumentException();
            }

            return CreateObject (ns, typeProperty, null, args, baseType);
        }

        /// <summary>
        /// Instantiates an object based on the typename found in the 
        /// configuration property of a specific namespace. 
        /// </summary>
        /// <param name="ns">Namespace where the property and type are located.</param>
        /// <param name="typeProperty">Property name that holds the typename.</param>
        /// <param name="assemblyProperty">Property to get the assembly from which to create the requested object.</param>
        /// <param name="baseType">Base typename determines if typeProperty is a valid typename.</param>
        /// <returns>The created object or null if unable to create.</returns>
        /// <exception cref="InstanceCreationException">
        /// If the instance cannot be created or the assembly cannot be loaded.
        /// </exception>
        /// <exception cref="ArgumentNullException">When any argument is null.</exception>
        /// <exception cref="ArgumentException">When ns or typeProperty or assemblyProperty is empty.</exception>
       
        public object CreateInstance (string ns, string typeProperty, 
            string assemblyProperty, Type baseType)
        {
            // This method will create the instance with the assembly name, 
            // and with constructor taking no arguments. It will simply delegate to 
            // CreateObject(ns, typeProperty, assemblyProperty, null, baseType).

            if (ns == null || typeProperty == null || assemblyProperty == null 
                || baseType == null)
            {
                throw new ArgumentNullException ();
            }

            if (ns.Trim().Length == 0 || typeProperty.Trim().Length == 0 || 
                assemblyProperty.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            return CreateObject (ns, typeProperty, assemblyProperty, null, baseType);
        }

        /// <summary>
        /// This method will create an instance of the requested type with the assembly name, 
        /// and with constructor taking arguments. 
        /// </summary>
        /// <param name="ns">Namespace where the property and type are located.</param>
        /// <param name="typeProperty">Property name that holds the typename.</param>
        /// <param name="assemblyProperty">Property to get the assembly from which to create the requested object.</param>
        /// <param name="args">Arguments to be passed to the constructor of the object to be created.</param>
        /// <param name="baseType">Base typename determines if typeProperty is a valid typename.</param>
        /// <returns>The created object or null if unable to create.</returns>
        /// <exception cref="InstanceCreationException">
        /// If the instance cannot be created or the assembly cannot be loaded.
        /// </exception>
        /// <exception cref="ArgumentNullException">When any argument is null.</exception>
        /// <exception cref="ArgumentException">When ns or typeProperty or assemblyProperty is empty.</exception>
        public object CreateInstance (string ns, string typeProperty, string assemblyProperty,
            object[] args, Type baseType)
        {


            if (ns == null || typeProperty == null || assemblyProperty == null || 
                args == null || baseType == null)
            {
                throw new ArgumentNullException ();
            }

            if (ns.Trim().Length == 0 || typeProperty.Trim().Length == 0 || 
                assemblyProperty.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            return CreateObject (ns, typeProperty, assemblyProperty, args, baseType);
        }

        /// <summary>
        /// Gets the value of a property, parsed based on parsing preferences specified.
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">Name of the property</param>
        /// <param name="preference">Rules for parsing.</param>
        /// <returns>The value of the property or null if not found.</returns>
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        /// <exception cref="ParsingPreferenceException">
        /// If the string value to be parsed doesn't meet the rules set in the parsing preference.
        /// </exception>
        public string GetValue (string ns, string property, IParsingPreference preference)
        {
            // Developer can set parsing preference such as if values should be 
            // trimmed before processing, if null/empty value should be allowed. 
            // An exception is thrown if a property  requested 
            // does not exist and/or is empty, etc. 
            // Different components within the same application should be able 
            // to maintain their own preferences.
            // This method will retrieve a single value as usual, then verify 
            // the return value with the IParsingPreference, which decides how to 
            // process the value according to user defined rules. 
            // It will call GetValue to retrieve the value first, and call the 
            // IParsingPreference.parse(value) to further process it. 
            // If no exception occurs during the processing, the parsed result 
            // will be returned, otherwise, the exception will be propagated

            if (ns == null || property == null ||  preference == null)
            {
                throw new ArgumentNullException();
            }

            if (ns.Trim().Length == 0 || property.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            return preference.Parse(GetValue (ns, property));
        }
        /// <summary>
        /// Gets the value of a property, parsed based on parsing preferences specified.
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">Name of the property</param>
        /// <param name="preference">Rules for parsing.</param>
        /// <returns>The values of the property or null if not found.</returns>
        /// <exception cref="NullReferenceException">
        /// If any argument is null.
        /// </exception>
        /// <exception cref="ParsingPreferenceException">
        /// If the string value to be parsed doesn't meet the rules set in the parsing preference.
        /// </exception>
        public string[] GetValues (string ns, string property, IParsingPreference preference)
        {
            //Developer can set parsing preference such as if values should be 
            //trimmed before processing, if null/empty value should be allowed, 
            //and if an exception should be thrown if a property is requested that 
            //does not exist and/or is empty, etc. Different components within the 
            //same application should be able to maintain their own preferences.
            //This method will retrieve an array of values as usual, then verify 
            //the each return value with the IParsingPreference, which decides 
            //how to process the value according to user defined rules. It will call 
            //GetValues to retrieve the value first, and call the 
            //IParsingPreference.parse(value) to further process each value. 
            //If no exception occurs during the processing, the parsed result will 
            //be returned, otherwise, the exception will be propagated.

            if (ns == null || property == null ||  preference == null)
            {
                throw new ArgumentNullException();
            }

            if (ns.Trim().Length == 0 || property.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            string[] values = GetValues(ns, property);
            ArrayList retValues = new ArrayList();

            foreach (string str in values)
            {
                retValues.Add(preference.Parse(str));
            }

            return (string[])retValues.ToArray(typeof(string));
        }

        /// <summary>
        /// Get the value of a property in a namespace as a specific datatype.
        /// </summary>
        /// <param name="ns">The namespace to look in.</param>
        /// <param name="property">The property name.</param>
        /// <param name="type">The specific datatype of the value to be retrieved.</param>
        /// <returns>The value or null if not found.</returns>
        /// <exception cref="NotSupportedException">
        /// If the retrieved property value cannot be converted into the specified type.
        /// </exception>
        public object GetValue (string ns, string property, Type type)
        {
            //Developer can easily retrieve properties as types other than 
            //string (such as bool, int, double, Enum, etc.). 
            //An application requests a parameter as a specific datatype and 
            //it will ensure that the property's value is parsable as that 
            //type before returning it (and then return it in a typesafe way, if possible).
        
            //It will return the property's value in the specified type; if the value 
            //can not be parsed/converted correctly, System.NotSupportedException 
            //will be thrown. This method will simply call the 
            //GetValue(ns, property, type, DefaultStringConverter.Instance) to convert 
            //the string using the DefaultStringConverter instance which can convert 
            //the string to primitive type using the IConvertible interface and Enum value.

            return GetValue (ns, property, type, DefaultStringConverter.Instance);
        }

        /// <summary>
        /// Get a set of values for a property as a specific datatype.
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">The property name in the namespace.</param>
        /// <param name="type">The datatype of the values to be retrieved.</param>
        /// <returns>
        /// An array of values or null if not found.
        /// </returns>
        /// <exception cref="NotSupportedException">
        /// If the any of retrieved values cannot be converted into the specified type.
        /// </exception>
        public Array GetValues (string ns, string property, Type type)
        {
            //Developer can easily retrieve properties as types other than string 
            //(such as bool, int, double, Enum, etc.) An application requests a 
            //parameter as a specific datatype and it will ensure that the property's 
            //value is parsable as that type before returning it 
            //(and then return it in a typesafe way, if possible).
            //It will return the property's values in the specified type, 
            //if the value can not be parsed/converted correctly, 
            //System.NotSupportedException will be thrown. 

            //This method will simply call the 
            //GetValues(ns, property, type, DefaultStringConverter.Instance) 
            //to convert each string in returned by GetValues(ns, property) method 
            //by DefaultStringConverter instance which can convert the string to 
            //primitive type using the IConvertible interface and Enum value.

            return GetValues (ns, property, type, DefaultStringConverter.Instance);
        }

        /// <summary>
        /// Get the value of a property in a namespace as a specific datatype.
        /// </summary>
        /// <param name="ns">The namespace to look in.</param>
        /// <param name="property">The property name.</param>
        /// <param name="type">The specific datatype of the value to be retrieved.</param>
        /// <param name="converter">
        /// Converter object to convert the retrieved value into the specified datatype.
        /// </param>
        /// <returns>The value or null if not found.</returns>
        /// <exception cref="FormatException">
        /// If the retrieved property value cannot be converted into the specified type.
        /// </exception>
        /// <exception cref="ArgumentNullException">If any argument is null.</exception>
        /// <exception cref="ArgumentException">If ns or property is empty.</exception>
        public object GetValue (string ns, string property, Type type, IStringConverter converter)
        {
            //Developer can easily retrieve properties as types other than string 
            //(such as bool, int, double, Enum, etc.) An application requests a 
            //parameter as a specific datatype and it will ensure that the property's 
            //value is parsable as that type before returning it 
            //(and then return it in a typesafe way, if possible).
            //It will return the property's value in the specified type, if the value 
            //can not be parsed/converted correctly, System.NotSupportedException 
            //will be thrown. 
            //This method will first get the value with GetValue(ns, property) method, 
            //and then call the converter.Convert(value, type) to convert the string into 
            //specific type, the client should cast the returned object into the specified 
            //type. For example, user can get the int value by calling: 
            //int value = (int) cm.GetValue(ns, property, typeof(int), DefaultStringConverter.Instance).

            object retValue;
            string val;

            if (ns == null || property == null || type == null || converter == null)
            {
                throw new ArgumentNullException();
            }

            if (ns.Trim().Length == 0 || property.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            try
            {
                val = GetValue (ns, property);
                retValue = converter.Convert(val, type);
            }
            catch (Exception e)
            {
                throw new FormatException("Cannot convert the requested value to " +
                    type.ToString(), e );
            }
            
            return retValue;

        }

        /// <summary>
        /// Get a set of values for a property as a specific datatype.
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">The property name in the namespace.</param>
        /// <param name="type">The datatype of the values to be retrieved.</param>
        /// <param name="converter">
        /// Converter object to convert the retrieved value into the specified datatype.
        /// </param>
        /// <returns>
        /// An array of values or null if not found.
        /// </returns>
        /// <exception cref="FormatException">
        /// If the any of retrieved values cannot be converted into the specified type.
        /// </exception>
        public Array GetValues (string ns, string property, Type type, IStringConverter converter)
        {
            // Developer can easily retrieve properties as types other than 
            // string (such as bool, int, double, Enum, etc.) An application 
            // requests a parameter as a specific datatype and it will ensure that 
            // the property's value is parsable as that type before returning it 
            // (and then return it in a typesafe way, if possible).
            // It will return the property's value in the specified type, if the 
            // value can not be parsed/converted correctly, System.NotSupportedException 
            // will be thrown. This method will first get the value with 
            // GetValues(ns, property) method, and then call the 
            // converter.Convert(value, type) against each value in the returned 
            // array to convert each string into specific type, the client should cast 
            // the returned object into the specified type. For example, user can get 
            // the int values by calling: 
            // int[] value = (int[]) cm.GetValues(ns, property, typeof(int), DefaultStringConverter.Instance).
            
            if (ns == null || property == null || type == null || converter == null)
            {
                throw new ArgumentNullException();
            }

            if (ns.Trim().Length == 0 || property.Trim().Length == 0)
            {
                throw new ArgumentException();
            }

            ArrayList retValues = new ArrayList();
            try
            {
                string[] vals  = GetValues (ns, property);
                foreach (string str in vals)
                {
                    retValues.Add (converter.Convert(str, type)); 
                }
            }
            catch (Exception e)
            {
                throw new FormatException("Cannot convert the requested value to " +
                    type.ToString(), e);
            }

            return retValues.ToArray(type);
        }

        /// <summary>
        /// A convenient method to return the property value as an int type. 
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">The property name.</param>
        /// <returns>
        /// The value as an int or null if not found.
        /// </returns>
        public int GetIntValue (string ns, string property)
        {
            //This method will first get the string property value, 
            //and then convert it to an int to return.
           
            return (int)GetValue (ns, property, typeof(int), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// A convenient method to return the property value as a double.
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">Property name.</param>
        /// <returns>
        /// The value as a double or null if not found.
        /// </returns>
        public double GetDoubleValue (string ns, string property)
        {
            // This method will first get the string property value, 
            // and then convert it to a double to return.

            return (double)GetValue (ns, property, typeof(double), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// A convenient method to return the property values as an int[] type. 
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">The property name.</param>
        /// <returns>
        /// The set of values as an int or null if not found.
        /// </returns>
        public int[] GetIntValues (string ns, string property)
        {
            //This method will first get the string property value, 
            //and then convert it to an int to return.

            ArrayList vals = new ArrayList(GetValues (ns, property, typeof(int), DefaultStringConverter.Instance));
            
            return (int[])vals.ToArray(typeof(int));
        }

        /// <summary>
        /// A convenient method to return the property values as double[].
        /// </summary>
        /// <param name="ns">Namespace to look in.</param>
        /// <param name="property">Property name.</param>
        /// <returns>
        /// The set of values as a double or null if not found.
        /// </returns>
        public double[] GetDoubleValues (string ns, string property)
        {
            // This method will first get the string property values, 
            // and then convert it to a double [] to return.

            ArrayList vals = new ArrayList(GetValues (ns, property, typeof(double), DefaultStringConverter.Instance));
            
            return (double[])vals.ToArray(typeof(double));
        }
    }
}
