// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER
// @version 2.0

using System;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// DefaultStringConverter implements the IStringConverter interface, 
    /// and it will be used by the ConfigManager by default if no IStringConverter 
    /// argument is specified when calling the GetValue/GetValues methods to get 
    /// a value as a specified type. This class is a singleton.
    /// </summary>
    public class DefaultStringConverter : IStringConverter
    {
        /// <summary>
        /// Represents the singleton static instance of the DefaultStringConverter.
        /// </summary>
        private static DefaultStringConverter instance = new DefaultStringConverter();

        /// <summary>
        /// Private constructor. Hidden since this class is a singleton.
        /// </summary>
        private DefaultStringConverter()
        {
        }

        /// <summary>
        /// Converts an input string to the specified type.
        /// </summary>
        /// <param name="val">Value to be converted.</param>
        /// <param name="type">The type to which val has to be converted.</param>
        /// <returns>
        /// The value to be converted into the specified type or its subclass.
        /// </returns>
        /// <exception cref="NotSupportedException">
        /// If the string value cannot be converted correctly.
        /// </exception>
        /// <exception cref="ArgumentNullException">
        /// If any of the arguments is null.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// If val is empty.
        /// </exception>
        public object Convert (string val, Type type)
        {
            object returnVal = null;

            if (val == null)
            {
                throw new ArgumentNullException ("val", "String to be converted cannot be null.");
            }

            if (val.Trim().Length == 0)
            {
                throw new ArgumentException ("String to be converted cannot be empty.", "val");
            }

            if (type == null)
            {
                throw new ArgumentNullException ("type", "Type cannot be null.");
            }

            try
            {
                if (type.IsEnum)
                {
                    returnVal = Enum.Parse(type, val);
                }
                else if (type == typeof(int))
                {
                    returnVal = int.Parse(val);
                }
                else if (type == typeof(double))
                {
                    returnVal = double.Parse(val);
                }
                else 
                {
                    returnVal = System.Convert.ChangeType(val, type);
                }
            }
            catch (Exception e) // cannot be converted
            {
                throw new NotSupportedException("The input value " + val + 
                    " cannot be converted to the type " + type.ToString(), e);
            }

            return returnVal;
        }

        /// <summary>
        /// Getter method to return the singleton DefaultStringConverter.
        /// </summary>
        public static DefaultStringConverter Instance
        {
            get
            {
                return instance;
            }
        }
    }
}
