// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.IO;
using System.Text;
using System.Collections;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// A configuration file handler for INI files.
    /// </summary>
    [Serializable]
    internal class IniFileHandler: FileHandler
    {
        /// <summary>
        /// Internal constructor.
        /// </summary>
        internal IniFileHandler()
        {
        }
        
        /// <summary>
        /// Determines whether the file is supported by the handler,
        /// usually by looking at the extension, but more complex 
        /// methods can be applied if needed. In this case
        /// the extension should be .ini.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <returns>Whether the file is supported or not.</returns>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public bool Supports(string filename)
        {
            return filename.ToLower().EndsWith(".ini");
        }
        
        /// <summary>
        /// Loads the namespaces and properties from the file into
        /// Namespace and Property objects.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="errorCallback">The error callback.</param>
        /// <returns>The populated Namespace objects</returns>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public Namespace[] LoadFromFile(string filename, 
                                        ErrorCallback errorCallback)
        {
            ArrayList namespaces = new ArrayList();
			FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read, 
				FileShare.Read);
            StreamReader reader = new StreamReader(fs);
            Namespace curNs = null;
            string line = reader.ReadLine();
            while (line != null)
            {
                if (line.StartsWith("#") || line.StartsWith(";"))
                {
                    // Do nothing, it's a comment line
                }
                else if (line.StartsWith("[") && line.EndsWith("]"))
                {
                    curNs = new Namespace(line.Substring(1, line.Length - 2));
                    namespaces.Add(curNs);
                }
                else if (line.IndexOf("=") != -1)
                {
                    if (curNs != null)
                    {
                        string propertyName = line.Substring(0, line.IndexOf("=")).Trim();
                        string valuesString = line.Substring(line.IndexOf("=")+1).Trim();
                        char[] delims = {';'};
                        string[] values = valuesString.Split(delims);
                        for (int i = 0; i < values.Length; i++)
                        {
                            values[i] = values[i].Trim();
                        }
                        curNs.AddProperty(new Property(propertyName, values), errorCallback);
                    }
                    else
                    {
                        // We've got values, but they're not inside a namespace
                        // It's possible that a different file handler implementation
                        // could create a default namespace in case namespaces are
                        // not desired within the config file.
                        throw new InvalidConfigFileException();
                    }
                }
                else if (line.Length > 0)
                {
                    // We have no idea what this line was supposed to mean
                    throw new InvalidConfigFileException();
                }
                line = reader.ReadLine();
            }
            reader.Close();
            fs.Close();
            return (Namespace[])namespaces.ToArray(typeof(Namespace));
        }
        
        /// <summary>
        /// Saves a set of Namespace objects and their properties to a file.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="namespaces">The set of namespaces.</param>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public void SaveFile(Namespace[] namespaces, string filename)
        {
            FileStream fs = new FileStream(filename, FileMode.Create);
            StreamWriter writer = new StreamWriter(fs);
            foreach (Namespace theNamespace in namespaces)
            {
                writer.WriteLine("[" + theNamespace.Name + "]");
                foreach (Property prop in theNamespace.Properties)
                {
                    StringBuilder propString = new StringBuilder(prop.Name + "=");
                    for (int i = 0; i < prop.Values.Length; i++)
                    {
                        if (i != prop.Values.Length - 1)
                        {
                            propString.Append(prop.Values[i] + ";");
                        }
                        else
                        {
                            propString.Append(prop.Values[i]);
                        }
                    }
                    writer.WriteLine(propString.ToString());
                }
                writer.WriteLine();
            }
            writer.Close();
            fs.Close();
        }
    }
}
