// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.Collections;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// A namespace groups logically a set of properties.
    /// As expected, the namespace contains a set of Property objects and
    /// properties and methods to access them.
    /// </summary>
    public class Namespace : MarshalByRefObject
    {
        /// <summary>
        /// The name of the namespace
        /// </summary>
        private string name = null;
        
        /// <summary>
        /// The properties contained in the namespace
        /// in the order in which they appear in the configuration file.
        /// </summary>
        private ArrayList properties = new ArrayList();
        
        /// <summary>
        /// A hash table mapping property names to Property objects
        /// to speed up the lookup of properties by name.
        /// </summary>
        private Hashtable propertiesLookup = Hashtable.Synchronized(new Hashtable());
        
        /// <summary>
        /// Is this namespace merged with another because of a name conflict?
        /// </summary>
        internal bool isMerged = false;
        
        /// <summary>
        /// Internal constructor.
        /// </summary>
        /// 
        /// <param name="name">The name of the namespace</param>
        internal Namespace(string name)
        {
            this.name = name;
        }
        
        /// <summary>
        /// Add a new property to the namespace.
        /// </summary>
        /// 
        /// <param name="errorCallback">The error callback.</param>
        /// <param name="property">The property to add</param>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        internal void AddProperty(Property property, 
                                  ErrorCallback errorCallback)
        {
            lock(this)
            {
                if (propertiesLookup.ContainsKey(property.Name))
                {
                    Property old = (Property)propertiesLookup[property.Name];
                    if (errorCallback != null)
                    {
                        ErrorAction errorAction = errorCallback.PropertyCollision(this, old, property);
                        if (errorAction == ErrorAction.ReplaceOld)
                        {
                            properties.Remove(old);
                            propertiesLookup.Remove(property.Name);
                            properties.Add(property);
                            propertiesLookup.Add(property.Name, property);
                        }
                        else if (errorAction == ErrorAction.KeepOld)
                        {
                            // Do nothing, the old one is already in place
                        }
                        else if (errorAction == ErrorAction.Merge)
                        {
                            // Merge the new values into the old ones
                            ArrayList newValues = new ArrayList();
                            newValues.AddRange(old.Values);
                            newValues.AddRange(property.Values);
                            old.Values = (string[])newValues.ToArray(typeof(string));
                            old.isMerged = true;
                        }
                        else if (errorAction == ErrorAction.ThrowException)
                        {
                            throw new PropertyClashException(property.Name + " clashed with a currently existing property.");
                        }
                    }
                    else
                    {
                        throw new PropertyClashException(property.Name + " clashed with a currently existing property.");
                    }
                }
                else
                {
                    properties.Add(property);
                    propertiesLookup.Add(property.Name, property);
                }
            }
        }
        
        /// <summary>
        /// The Name property.
        /// </summary>
        /// <value>
        /// The name of the namespace.
        /// </value>
        public string Name
        {
            get
            {
                return name;
            }
        }
        
        /// <summary>
        /// The Properties property.
        /// </summary>
        /// <value>
        /// The array of properties owned by the namespace.
        /// </value>
        public Property[] Properties
        {
            get 
            {
                return (Property[])properties.ToArray(typeof(Property));
            }
        }
        
        /// <summary>
        /// Get the property with the specified name.
        /// </summary>
        /// 
        /// <param name="name">The name of the property</param>
        /// <returns>The property or null if not found.</returns>
        /// 
        /// <exception cref="NullReferenceException">
        /// if name is null
        /// </exception>
        public Property GetProperty(string name)
        {
            if (name == null)
            {
                throw new NullReferenceException();
            }
            return (Property)propertiesLookup[name];
        }
        
        /// <summary>
        /// Shortcut for accessing a single value property by its name.
        /// Same as GetProperty(string name).Value
        /// </summary>
        /// 
        /// <param name="name">The name of the property</param>
        /// 
        /// <exception cref="NullReferenceException">
        /// if name is null
        /// </exception>
        public string this[string name]
        {
            get
            {
                if (name == null)
                {
                    throw new NullReferenceException();
                }
                return GetProperty(name).Value;
            }
            set
            {
                if (name == null)
                {
                    throw new NullReferenceException();
                }
                GetProperty(name).Value = value;
            }
        }
    }
}
