// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER
// @version 2.0

using System;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// ParsingPreference implements the IParsingPreference interface. 
    /// It provides some basic rules like judging the string value can be null, 
    /// empty string, or the spaces in the value should be trimmed. 
    /// Client should provide its own implementation if more preference 
    /// rules are required.
    /// </summary>
    public class ParsingPreference : IParsingPreference
    {
        /// <summary>
        /// Represents whether a null string value as input is allowed or not.
        /// </summary>
        private bool allowNull = true;

        /// <summary>
        /// Represents whether an empty string value is allowed as an input or not.
        /// </summary>
        private bool allowEmpty = true;

        /// <summary>
        /// Represents the whether spaces in the input string should be trimmed or not.
        /// </summary>
        private bool trimSpace = false;

        /// <summary>
        /// Determines whether a null string input is allowed or not. 
        /// Value is true by default. 
        /// If AllowNull is false, the string to be parsed should always be non-null.
        /// </summary>
        public bool AllowNull
        {
            get
            {
                return allowNull;
            }
            set
            {
                allowNull = value;
            }
        }

        /// <summary>
        /// Determines whether an empty string input is allowed or not. 
        /// Value is true by default.
        /// If allowEmpty is false, the input string should always be non-empty.
        /// </summary>
        public bool AllowEmpty
        {
            get
            {
                return allowEmpty;
            }
            set
            {
                allowEmpty = value;
            }
        }

        /// <summary>
        /// Determines if spaces in the input string should be trimmed or not. 
        /// Value is false by default. If the trimSpace is true, then white spaces
        /// in the input string will be trimmed and returned by the Parse method.
        /// </summary>
        public bool TrimSpace
        {
            get
            {
                return trimSpace;
            }
            set
            {
                trimSpace = value;
            }
        }
        
        /// <summary>
        /// Parses the string value according to user defined rules, and returns 
        /// the processed string to user.  
        /// This method is called by GetValue/GetValues methods of ConfigurationManager
        /// with a IParsingPreference argument to parse the string value retrieved.
        /// </summary>
        /// <param name="val">String to be parsed.</param>
        /// <returns>Parsed string.</returns>
        /// <exception cref="ParsingPreferenceException">
        /// If the string value violates some rule, for example if the string
        /// can not be null and the one passed in is null.
        /// </exception>
        public string Parse (string val)
        {
            string parsedString = val;

            if (!allowNull && val == null)
            {
                throw new ParsingPreferenceException ("String value to be parsed cannot be null.");
            }

            if (val != null)
            {
                if (!allowEmpty && val.Trim().Length == 0)
                {
                    throw new ParsingPreferenceException ("String value to be parsed cannot be empty.");
                }

                if (trimSpace)
                {
                    parsedString = val.Trim();
                }
            }


            return parsedString;
        }


        /// <summary>
        /// Default constructor.
        /// </summary>
        public ParsingPreference()
        {
            //do nothing here.
        }
    }
}
