// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER
// @version 2.0

using System;
using System.Collections;
using System.Xml;
using System.Reflection;
using System.IO;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// PluggableHandler  will be used by ConfigFile to load the config values 
    /// from pluggable datasources. It supports a kind of file with 'mxml' file extension 
    /// (which is actually in xml format), and will create an IDataHandler instance 
    /// dynamically using necessary data defined in the file using reflection. 
    /// The created IDataHandler instance is responsible for the loading/saving of 
    /// the configuration values.
    /// Whenever a client needs to communicate with a different kind of datasource, 
    /// it has to provide an implementation of IDataHandler, and create an 
    /// mxml file which will be used by the PluggableHandler to instantiate it dynamically.
    /// </summary>
    public class PluggableHandler : FileHandler
    {
        /// <summary>
        /// Private method to create an IDataHandler implementation from an mxml file.
        /// </summary>
        /// <param name="filename">
        /// An mxml file that contains the appropriate data for creating an 
        /// implementation of IDataHandler.
        /// </param>
        /// <returns>
        /// An IDataHandler object or null if an error occurs.
        /// </returns>
        /// <exception cref="InvalidConfigFileException">
        /// If filename is an invalid configuration file.
        /// </exception>
        /// <exception cref="DataHandleException">
        /// If the IDataHandler instance cannot be created from the information
        /// given in the mxml file.
        /// </exception>
        private IDataHandler CreateInstanceFromFile (string filename)
        {
            string dataHandlerClassName = null;
            string assemblyName  = null;
            IDictionary datasourceInfo = new Hashtable();


            // first, attempt to load the contents of the mxml file
            try
            {
                XmlDocument mxmlDoc = new XmlDocument();
                mxmlDoc.Load(filename);

                // read the value of <Class>MyDataHandler</Class>
                // this class is the implementation of IDataHandler
                XmlNode nodeClass = mxmlDoc.GetElementsByTagName("Class")[0];
                dataHandlerClassName =  nodeClass.InnerText.Trim();

                // read the value of <Assembly>MyDataHandlerAssembly.dll</Assembly>
                // this is the assembly where the IDataHandler implementation resides.
                XmlNode nodeAssembly = mxmlDoc.GetElementsByTagName("Assembly")[0];
                assemblyName = nodeAssembly.InnerText.Trim();

                // read the information about the location of the datasource.
                // <Properties> 
                //      <Property value="connUrl">
                //           <Value>User Name=sa;Pwd=sa;Server=myServer</Value>
                //      </Property>
                // </Properties>
                XmlNode nodeProperties = mxmlDoc.GetElementsByTagName("Properties")[0];

                XmlNodeList nodePropertyList = nodeProperties.ChildNodes;

                // read the constructor's name-value pair argument which gives 
                // information about the datasource schema.
                foreach (XmlNode property in nodePropertyList)
                {
                    string propertyName = property.Attributes["name"].InnerText;
                    string propertyValue = property["Value"].InnerText;
                    datasourceInfo.Add(propertyName, propertyValue);
                }
            }
            catch (XmlException xmlException)
            {
                throw new InvalidConfigFileException(xmlException.Message);
            }

            //next, attempt to create the IDataHandler implementation
            Type t = null;

            try
            {
                Assembly a = null;

                if (assemblyName.Trim().Length != 0)
                {
                    a = Assembly.LoadFrom(assemblyName);
                    t = a.GetType(dataHandlerClassName, true, true);
                }
                else
                {
                    t = Type.GetType(dataHandlerClassName, true, true);
                }
            }
            catch (Exception e)
            {
                throw new DataHandleException (e.Message);
            }
                
            if (typeof(IDataHandler).IsAssignableFrom(t))
            {
                object [] args = new object[] {datasourceInfo};
                object dataHandlerInst = Activator.CreateInstance(t, args);
                IDataHandler dataHandler  = dataHandlerInst as IDataHandler;

                return dataHandler;
            }
            else
            {
                throw new DataHandleException(dataHandlerClassName + 
                    " does not implement IDataHandler.");
            }
        }

        /// <summary>
        /// Internal empty constructor.
        /// </summary>
        internal PluggableHandler()
        {
        }

        /// <summary>
        /// Determines whether the file is supported by the handler,
        /// usually by looking at the extension, but more complex 
        /// methods can be applied if needed. In this case
        /// the extension should be .mxml.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <returns>Whether the file is supported or not.</returns>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public bool Supports(string filename)
        {
            return filename.ToLower().EndsWith(".mxml");
        }

        /// <summary>
        /// Load the namespaces from the IDataHandler instance created from the necessary 
        /// data defined in the mxml file, into Namespace and Property objects.
        /// </summary>
        /// 
        /// <param name="filename">The name of the mxml file.</param>
        /// <param name="errorCallback">The error callback.</param>
        /// <returns>The populated Namespace objects</returns>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the mxml configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public Namespace[] LoadFromFile(string filename, 
            ErrorCallback errorCallback)
        {
            // This method will first create the IDataHandler instance using 
            // reflection with the classname and assembly name defined in the file, 
            // and some necessary arguments to instantiate the instance. 
            // Finally it will call the IDataHandler.Load(errorCallback) to load 
            // the namespaces from the datasource.

                IDataHandler dataHandler = CreateInstanceFromFile (filename);
                IList ns = dataHandler.Load(errorCallback);
                ArrayList namespaces = new ArrayList(ns);

                return (Namespace[])namespaces.ToArray(typeof(Namespace));
        }



        /// <summary>
        /// Saves a set of Namespace objects and their properties into 
        /// the datasource related to the IDataHandler created with the values 
        /// defined in the mxml file.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="namespaces">The set of namespaces.</param>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public void SaveFile(Namespace[] namespaces, string filename)
        {
            //This method will first create the IDataHandler instance with the data 
            //from the mxml file and some necessary arguments to instantiate it, 
            //and then save the namespaces by calling IDataHandler.Save method.

                IDataHandler dataHandler = CreateInstanceFromFile(filename);

                dataHandler.Save(namespaces);
        }
        
    }
}
