// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.IO;
using System.Collections;
using System.Xml;
    
namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// A configuration file handler for XML files as they are defined
    /// in the component specification.
    /// </summary>
    [Serializable]
    internal class XmlFileHandler: FileHandler
    {
        /// <summary>
        /// Internal constructor.
        /// </summary>
        internal XmlFileHandler()
        {
        }
        
        /// <summary>
        /// Determines whether the file is supported by the handler,
        /// usually by looking at the extension, but more complex 
        /// methods can be applied if needed. In this case
        /// the extension should be .xml.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <returns>Whether the file is supported or not.</returns>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public bool Supports(string filename)
        {
            return filename.ToLower().EndsWith(".xml");
        }
        
        /// <summary>
        /// Loads the namespaces and properties from the file into
        /// Namespace and Property objects.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="errorCallback">The error callback.</param>
        /// <returns>The populated Namespace objects</returns>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public Namespace[] LoadFromFile(string filename, 
                                        ErrorCallback errorCallback)
        {
            try
            {
                ArrayList namespaces = new ArrayList();
                XmlTextReader reader = new XmlTextReader(filename);
                Namespace curNs = null;
                while (reader.Read())
                {
                    if (reader.Name.ToLower() == "namespace" && reader.IsStartElement())
                    {
                        string namespaceName = (string)reader.GetAttribute("name");
                        curNs = new Namespace(namespaceName);
                        namespaces.Add(curNs);
                        while (reader.Read())
                        {
                            if (reader.Name.ToLower() == "property" && reader.IsStartElement())
                            {
                                string propertyName = (string)reader.GetAttribute("name");
                                ArrayList valueList = new ArrayList();
                                while (reader.Read())
                                {
                                    if (reader.Name.ToLower() == "value" && reader.IsStartElement())
                                    {
                                        reader.Read();
                                        valueList.Add(reader.Value);
                                    }
                                    else if (reader.Name.ToLower() == "property" && reader.NodeType == XmlNodeType.EndElement)
                                    {
                                        break;
                                    }
                                }
                                string[] values = (string[])valueList.ToArray(typeof(string));
                                curNs.AddProperty(new Property(propertyName, values), errorCallback);
                            }
                            else if (reader.Name.ToLower() == "namespace" && reader.NodeType == XmlNodeType.EndElement)
                            {
                                break;
                            }
                        }
                    }
                    else if (reader.Name.ToLower() == "ConfigManager" && reader.NodeType == XmlNodeType.EndElement)
                    {
                        break;
                    }
                }
                reader.Close();
                return (Namespace[])namespaces.ToArray(typeof(Namespace));
            }
            catch (XmlException)
            {
                throw new InvalidConfigFileException();
            }
        }
        
        /// <summary>
        /// Saves a set of Namespace objects and their properties to a file.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="namespaces">The set of namespaces.</param>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        public void SaveFile(Namespace[] namespaces, string filename)
        {
            XmlTextWriter writer = new XmlTextWriter(filename, System.Text.Encoding.UTF8);
            writer.Formatting = Formatting.Indented;
            writer.WriteStartDocument();
            writer.WriteStartElement("ConfigManager");
            foreach (Namespace theNamespace in namespaces)
            {
                writer.WriteStartElement("namespace");
                writer.WriteAttributeString("name", theNamespace.Name);
                foreach (Property prop in theNamespace.Properties)
                {
                    writer.WriteStartElement("property");
                    writer.WriteAttributeString("name", prop.Name);
                    for (int i = 0; i < prop.Values.Length; i++)
                    {
                        writer.WriteElementString("value", prop.Values[i]);
                    }
                    writer.WriteEndElement();
                }
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteEndDocument();
            writer.Close();
        }
    }
}
