/*
	Copyright c 2003, TopCoder, Inc. All rights reserved.
	Author pzhao
*/

using System;
using NUnit.Framework;

using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Util.ConfigurationManager.AccuracyTests
{
    /// <summary>
    /// Accuracy test cases related to the namespace class.
    /// </summary>
    [TestFixture]
    public class ConfigManagerAccuracyTest
    {
        /// <summary>
        /// The configuration manager instance.
        /// </summary>
        private ConfigManager cm = null;

        /// <summary>
        /// The configuration file instance.
        /// </summary>
        private ConfigFile cf = null;

        /// <summary>
        /// Initialization
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            cm = ConfigManager.GetInstance();
        }

        /// <summary>
        /// Test get instance.
        /// </summary>
        [Test]
        public void TestGetInstance()
        {
            Assertion.AssertNotNull(cm);
        }

        /// <summary>
        /// Test LoadFile.
        /// </summary>
        [Test]
        public void TestLoadFile()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");
            Assertion.AssertEquals(cf.Namespaces.Length, 2);
            Assertion.AssertEquals(cf.Namespaces[0].Name, "AccuracyNamespace1");
            Assertion.AssertEquals(cf.Namespaces[1].Name, "AccuracyNamespace2");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");

            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.xml", new DefaultErrorCallback());
            Assertion.AssertEquals(cf.Namespaces.Length, 2);
            Assertion.AssertEquals(cf.Namespaces[0].Name, "AccuracyNamespace3");
            Assertion.AssertEquals(cf.Namespaces[1].Name, "AccuracyNamespace4");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.xml");

            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.xml", "NewNamespace1");
            Assertion.AssertEquals(cf.Namespaces.Length, 1);
            Assertion.AssertEquals(cf.Namespaces[0].Name, "NewNamespace1");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.xml");

            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.xml", "NewNamespace2", new DefaultErrorCallback());
            Assertion.AssertEquals(cf.Namespaces.Length, 1);
            Assertion.AssertEquals(cf.Namespaces[0].Name, "NewNamespace2");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.xml");
        }

        /// <summary>
        /// Test get namespace.
        /// </summary>
        [Test]
        public void TestGetNamespace()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");

            Assertion.AssertEquals(cm.GetNamespace("AccuracyNamespace1").Name, "AccuracyNamespace1");
            Assertion.AssertEquals(cm.GetNamespace("AccuracyNamespace1").Properties.Length, 3);
            Assertion.AssertEquals(cm.GetNamespace("AccuracyNamespace2").Name, "AccuracyNamespace2");
            Assertion.AssertEquals(cm.GetNamespace("AccuracyNamespace2").Properties.Length, 3);

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
        }

        /// <summary>
        /// Test get property.
        /// </summary>
        [Test]
        public void TestGetProperty()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");

            Assertion.AssertEquals(cm.GetProperty("AccuracyNamespace1", "AccuracyProp1").Value, "AccuracyValue1");
            Assertion.AssertEquals(cm.GetProperty("AccuracyNamespace1", "AccuracyProp2").Value, "AccuracyValue2");
            Assertion.AssertEquals(cm.GetProperty("AccuracyNamespace1", "AccuracyProp3").Value, "AccuracyValue3");
            Assertion.AssertEquals(cm.GetProperty("AccuracyNamespace2", "AccuracyProp1").Value, "AccuracyValue3");
            Assertion.AssertEquals(cm.GetProperty("AccuracyNamespace2", "AccuracyProp2").Value, "AccuracyValue2");
            Assertion.AssertEquals(cm.GetProperty("AccuracyNamespace2", "AccuracyProp3").Value, "AccuracyValue1");

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
        }

        /// <summary>
        /// Test get/set value.
        /// </summary>
        [Test]
        public void TestValue()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");

            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp1"), "AccuracyValue1");
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp2"), "AccuracyValue2");
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp3"), "AccuracyValue3");
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace2", "AccuracyProp1"), "AccuracyValue3");
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace2", "AccuracyProp2"), "AccuracyValue2");
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace2", "AccuracyProp3"), "AccuracyValue1");

            Assertion.Assert(cm.SetValue("AccuracyNamespace1", "AccuracyProp1", "NewValue1"));
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp1"), "NewValue1");
            Assertion.Assert(cm.SetValue("AccuracyNamespace1", "AccuracyProp2", "NewValue2"));
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp2"), "NewValue2");
            Assertion.Assert(cm.SetValue("AccuracyNamespace1", "AccuracyProp3", "NewValue3"));
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp3"), "NewValue3");

            Assertion.Assert(!cm.SetValue("AccuracyNamespace1", "FakedProp1", "NewValue1"));
            Assertion.Assert(!cm.SetValue("FakedNamespace1", "AccuracyProp1", "NewValue1"));

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
        }

        /// <summary>
        /// Test refresh.
        /// </summary>
        [Test]
        public void TestRefresh()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");

            Assertion.Assert(cm.SetValue("AccuracyNamespace1", "AccuracyProp1", "NewValue1"));
            cm.Refresh();
            Assertion.AssertEquals(cm.GetValue("AccuracyNamespace1", "AccuracyProp1"), "AccuracyValue1");

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
        }

        /// <summary>
        /// Test get/set values.
        /// </summary>
        [Test]
        public void TestValues()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");

            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace1", "AccuracyProp1").Length, 1);
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace1", "AccuracyProp2").Length, 2);
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace1", "AccuracyProp3").Length, 3);
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace2", "AccuracyProp1").Length, 3);
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace2", "AccuracyProp2").Length, 2);
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace2", "AccuracyProp3").Length, 1);

            Assertion.Assert(cm.SetValues("AccuracyNamespace1", "AccuracyProp1", new String[] {"NewValue1"}));
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace1", "AccuracyProp1").Length, 1);
            Assertion.Assert(cm.SetValues("AccuracyNamespace1", "AccuracyProp2", new String[] {"NewValue1"}));
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace1", "AccuracyProp2").Length, 1);
            Assertion.Assert(cm.SetValues("AccuracyNamespace1", "AccuracyProp3", new String[] {"NewValue1"}));
            Assertion.AssertEquals(cm.GetValues("AccuracyNamespace1", "AccuracyProp3").Length, 1);

            Assertion.Assert(!cm.SetValues("AccuracyNamespace1", "FakedProp1", new String[] {"NewValue1"}));
            Assertion.Assert(!cm.SetValues("FakedNamespace1", "AccuracyProp1", new String[] {"NewValue1"}));

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
        }

        /// <summary>
        /// Test get config file.
        /// </summary>
        [Test]
        public void TestGetConfigFile()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.xml");

            Assertion.AssertEquals(cm.GetConfigFile(@"..\..\test_files\AccuracyTest.ini").Namespaces.Length, 2);
            Assertion.AssertEquals(cm.GetConfigFile(@"..\..\test_files\AccuracyTest.xml").Namespaces.Length, 2);

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.xml");
        }

        /// <summary>
        /// Test indexer.
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");
            Assertion.AssertEquals(cm["AccuracyNamespace1"].Name, "AccuracyNamespace1");
            Assertion.AssertEquals(cm["AccuracyNamespace1"].Properties.Length, 3);
            Assertion.AssertEquals(cm["AccuracyNamespace2"].Name, "AccuracyNamespace2");
            Assertion.AssertEquals(cm["AccuracyNamespace2"].Properties.Length, 3);
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
        }

        /// <summary>
        /// Test get namespaces.
        /// </summary>
        [Test]
        public void TestGetNamespaces()
        {
            int len = cm.Namespaces.Length;
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");
            Assertion.AssertEquals(cm.Namespaces.Length - len, 2);
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.xml");
            Assertion.AssertEquals(cm.Namespaces.Length - len, 4);

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.xml");
        }

        /// <summary>
        /// Test get config files.
        /// </summary>
        [Test]
        public void TestGetConfigFiles()
        {
            int len = cm.ConfigFiles.Length;
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.ini");
            Assertion.AssertEquals(cm.ConfigFiles.Length - len, 1);
            cf = cm.LoadFile(@"..\..\test_files\AccuracyTest.xml");
            Assertion.AssertEquals(cm.ConfigFiles.Length - len, 2);

            cm.RemoveFile(@"..\..\test_files\AccuracyTest.ini");
            cm.RemoveFile(@"..\..\test_files\AccuracyTest.xml");
        }
    }
}