/*
Copyright (c) 2004, TopCoder, Inc. All rights reserved.
Author: WishingBone
*/

using System;
using NUnit.Framework;

namespace TopCoder.Util.ConfigurationManager.FailureTests
{

    /// <summary>
    /// Failure test fixture for ConfigManager in version 2.0.
    /// </summary>
    [TestFixture]
    public class ConfigManagerV2FailureTests
    {

        /// <summary>
        /// The test file location.
        /// </summary>
        private const string FILE = @"../../test_files/failure_v2.xml";

        /// <summary>
        /// The test namespace.
        /// </summary>
        private const string NAMESPACE = "TopCoder.Util.ConfigurationManager.FailureTests.ConfigManagerV2FailureTests";

        /// <summary>
        /// The test property name.
        /// </summary>
        private const string PROPERTY = "prop_name";

        /// <summary>
        /// The test assembly name.
        /// </summary>
        private const string ASSEMBLY = "Mscorlib";

        /// <summary>
        /// The configuration manager singleton.
        /// </summary>
        private ConfigManager cm = ConfigManager.GetInstance();

        /// <summary>
        /// Load the configuration file.
        /// </summary>
        [TestFixtureSetUp]
        public void LoadFile()
        {
            cm.LoadFile(FILE);
        }

        /// <summary>
        /// Unload the configuration file.
        /// </summary>
        [TestFixtureTearDown]
        public void UnloadFile()
        {
            cm.RemoveFile(FILE);
        }

        /// <summary>
        /// CreateInstance() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance1_NullNamespace()
        {
            cm.CreateInstance(null, PROPERTY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance1_NullProperty()
        {
            cm.CreateInstance(NAMESPACE, null, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance1_NullType()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, null);
        }

        /// <summary>
        /// CreateInstance() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance1_EmptyNamespace()
        {
            cm.CreateInstance(string.Empty, PROPERTY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance1_EmptyProperty()
        {
            cm.CreateInstance(NAMESPACE, string.Empty, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with incompatible type.
        /// </summary>
        [Test, ExpectedException(typeof(InstanceCreationException))]
        public void CreateInstance1_IncompatibleType()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "System.Int32");
            cm.CreateInstance(NAMESPACE, PROPERTY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance2_NullNamespace()
        {
            cm.CreateInstance(null, PROPERTY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance2_NullProperty()
        {
            cm.CreateInstance(NAMESPACE, null, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null args.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance2_NullArgs()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, (object[]) null, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance2_NullType()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, new object[1], null);
        }

        /// <summary>
        /// CreateInstance() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance2_EmptyNamespace()
        {
            cm.CreateInstance(string.Empty, PROPERTY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance2_EmptyProperty()
        {
            cm.CreateInstance(NAMESPACE, string.Empty, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with incompatible type.
        /// </summary>
        [Test, ExpectedException(typeof(InstanceCreationException))]
        public void CreateInstance2_IncompatibleType()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "System.Int32");
            cm.CreateInstance(NAMESPACE, PROPERTY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance3_NullNamespace()
        {
            cm.CreateInstance(null, PROPERTY, ASSEMBLY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance3_NullProperty()
        {
            cm.CreateInstance(NAMESPACE, null, ASSEMBLY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null assembly.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance3_NullAssembly()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, (string) null, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance3_NullType()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, ASSEMBLY, null);
        }

        /// <summary>
        /// CreateInstance() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance3_EmptyNamespace()
        {
            cm.CreateInstance(string.Empty, PROPERTY, ASSEMBLY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance3_EmptyProperty()
        {
            cm.CreateInstance(NAMESPACE, string.Empty, ASSEMBLY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with empty assembly.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance3_EmptyAssembly()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, string.Empty, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with incompatible type.
        /// </summary>
        [Test, ExpectedException(typeof(InstanceCreationException))]
        public void CreateInstance3_IncompatibleType()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "System.Int32");
            cm.CreateInstance(NAMESPACE, PROPERTY, ASSEMBLY, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance4_NullNamespace()
        {
            cm.CreateInstance(null, PROPERTY, ASSEMBLY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance4_NullProperty()
        {
            cm.CreateInstance(NAMESPACE, null, ASSEMBLY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null assembly.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance4_NullAssembly()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, null, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null args.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance4_NullArgs()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, ASSEMBLY, null, typeof(string));
        }

        /// <summary>
        /// CreateInstance() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void CreateInstance4_NullType()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, ASSEMBLY, new object[1], null);
        }

        /// <summary>
        /// CreateInstance() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance4_EmptyNamespace()
        {
            cm.CreateInstance(string.Empty, PROPERTY, ASSEMBLY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance4_EmptyProperty()
        {
            cm.CreateInstance(NAMESPACE, string.Empty, ASSEMBLY, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with empty assembly.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CreateInstance4_EmptyAssembly()
        {
            cm.CreateInstance(NAMESPACE, PROPERTY, string.Empty, new object[1], typeof(string));
        }

        /// <summary>
        /// CreateInstance() with incompatible type.
        /// </summary>
        [Test, ExpectedException(typeof(InstanceCreationException))]
        public void CreateInstance4_IncompatibleType()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "System.Int32");
            cm.CreateInstance(NAMESPACE, PROPERTY, ASSEMBLY, new object[1], typeof(string));
        }

        /// <summary>
        /// GetValue() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue1_NullNamespace()
        {
            cm.GetValue(null, PROPERTY, new ParsingPreference());
        }

        /// <summary>
        /// GetValue() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue1_NullProperty()
        {
            cm.GetValue(NAMESPACE, null, new ParsingPreference());
        }

        /// <summary>
        /// GetValue() with null preference.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue1_NullPreference()
        {
            cm.GetValue(NAMESPACE, PROPERTY, (IParsingPreference) null);
        }

        /// <summary>
        /// GetValue() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValue1_EmptyNamespace()
        {
            cm.GetValue(string.Empty, PROPERTY, new ParsingPreference());
        }

        /// <summary>
        /// GetValue() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValue1_EmptyProperty()
        {
            cm.GetValue(NAMESPACE, string.Empty, new ParsingPreference());
        }

        /// <summary>
        /// GetValues() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues1_NullNamespace()
        {
            cm.GetValues(null, PROPERTY, new ParsingPreference());
        }

        /// <summary>
        /// GetValues() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues1_NullProperty()
        {
            cm.GetValues(NAMESPACE, null, new ParsingPreference());
        }

        /// <summary>
        /// GetValues() with null preference.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues1_NullPreference()
        {
            cm.GetValues(NAMESPACE, PROPERTY, (IParsingPreference) null);
        }

        /// <summary>
        /// GetValues() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValues1_EmptyNamespace()
        {
            cm.GetValues(string.Empty, PROPERTY, new ParsingPreference());
        }

        /// <summary>
        /// GetValues() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValues1_EmptyProperty()
        {
            cm.GetValues(NAMESPACE, string.Empty, new ParsingPreference());
        }

        /// <summary>
        /// GetValue() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue2_NullNamespace()
        {
            cm.GetValue(null, PROPERTY, typeof(string));
        }

        /// <summary>
        /// GetValue() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue2_NullProperty()
        {
            cm.GetValue(NAMESPACE, null, typeof(string));
        }

        /// <summary>
        /// GetValue() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue2_NullType()
        {
            cm.GetValue(NAMESPACE, PROPERTY, (Type) null);
        }

        /// <summary>
        /// GetValue() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValue2_EmptyNamespace()
        {
            cm.GetValue(string.Empty, PROPERTY, typeof(string));
        }

        /// <summary>
        /// GetValue() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValue2_EmptyProperty()
        {
            cm.GetValue(NAMESPACE, string.Empty, typeof(string));
        }

        /// <summary>
        /// GetValues() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues2_NullNamespace()
        {
            cm.GetValues(null, PROPERTY, typeof(string));
        }

        /// <summary>
        /// GetValues() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues2_NullProperty()
        {
            cm.GetValues(NAMESPACE, null, typeof(string));
        }

        /// <summary>
        /// GetValues() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues2_NullType()
        {
            cm.GetValues(NAMESPACE, PROPERTY, (Type) null);
        }

        /// <summary>
        /// GetValues() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValues2_EmptyNamespace()
        {
            cm.GetValues(string.Empty, PROPERTY, typeof(string));
        }

        /// <summary>
        /// GetValues() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValues2_EmptyProperty()
        {
            cm.GetValues(NAMESPACE, string.Empty, typeof(string));
        }

        /// <summary>
        /// GetValue() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue3_NullNamespace()
        {
            cm.GetValue(null, PROPERTY, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValue() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue3_NullProperty()
        {
            cm.GetValue(NAMESPACE, null, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValue() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue3_NullType()
        {
            cm.GetValue(NAMESPACE, PROPERTY, null, DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValue() with null converter.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValue3_NullConverter()
        {
            cm.GetValue(NAMESPACE, PROPERTY, typeof(string), null);
        }

        /// <summary>
        /// GetValue() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValue3_EmptyNamespace()
        {
            cm.GetValue(string.Empty, PROPERTY, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValue() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValue3_EmptyProperty()
        {
            cm.GetValue(NAMESPACE, string.Empty, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValues() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues3_NullNamespace()
        {
            cm.GetValues(null, PROPERTY, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValues() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues3_NullProperty()
        {
            cm.GetValues(NAMESPACE, null, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValues() with null type.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues3_NullType()
        {
            cm.GetValues(NAMESPACE, PROPERTY, null, DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValues() with null converter.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetValues3_NullConverter()
        {
            cm.GetValues(NAMESPACE, PROPERTY, typeof(string), null);
        }

        /// <summary>
        /// GetValues() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValues3_EmptyNamespace()
        {
            cm.GetValues(string.Empty, PROPERTY, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetValues() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetValues3_EmptyProperty()
        {
            cm.GetValues(NAMESPACE, string.Empty, typeof(string), DefaultStringConverter.Instance);
        }

        /// <summary>
        /// GetIntValue() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetIntValue_NullNamespace()
        {
            cm.GetIntValue(null, PROPERTY);
        }

        /// <summary>
        /// GetIntValue() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetIntValue_NullProperty()
        {
            cm.GetIntValue(NAMESPACE, null);
        }

        /// <summary>
        /// GetIntValue() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetIntValue_EmptyNamespace()
        {
            cm.GetIntValue(string.Empty, PROPERTY);
        }

        /// <summary>
        /// GetIntValue() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetIntValue_EmptyProperty()
        {
            cm.GetIntValue(NAMESPACE, string.Empty);
        }

        /// <summary>
        /// GetIntValue() with invalid format.
        /// </summary>
        [Test, ExpectedException(typeof(FormatException))]
        public void GetIntValue_InvalidFormat()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "twelve");
            cm.GetIntValue(NAMESPACE, PROPERTY);
        }

        /// <summary>
        /// GetDoubleValue() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetDoubleValue_NullNamespace()
        {
            cm.GetDoubleValue(null, PROPERTY);
        }

        /// <summary>
        /// GetDoubleValue() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetDoubleValue_NullProperty()
        {
            cm.GetDoubleValue(NAMESPACE, null);
        }

        /// <summary>
        /// GetDoubleValue() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetDoubleValue_EmptyNamespace()
        {
            cm.GetDoubleValue(string.Empty, PROPERTY);
        }

        /// <summary>
        /// GetDoubleValue() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetDoubleValue_EmptyProperty()
        {
            cm.GetDoubleValue(NAMESPACE, string.Empty);
        }

        /// <summary>
        /// GetDoubleValue() with invalid format.
        /// </summary>
        [Test, ExpectedException(typeof(FormatException))]
        public void GetDoubleValue_InvalidFormat()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "twelve");
            cm.GetDoubleValue(NAMESPACE, PROPERTY);
        }

        /// <summary>
        /// GetIntValues() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetIntValues_NullNamespace()
        {
            cm.GetIntValues(null, PROPERTY);
        }

        /// <summary>
        /// GetIntValues() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetIntValues_NullProperty()
        {
            cm.GetIntValues(NAMESPACE, null);
        }

        /// <summary>
        /// GetIntValues() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetIntValues_EmptyNamespace()
        {
            cm.GetIntValues(string.Empty, PROPERTY);
        }

        /// <summary>
        /// GetIntValues() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetIntValues_EmptyProperty()
        {
            cm.GetIntValues(NAMESPACE, string.Empty);
        }

        /// <summary>
        /// GetIntValues() with invalid format.
        /// </summary>
        [Test, ExpectedException(typeof(FormatException))]
        public void GetIntValues_InvalidFormat()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "twelve");
            cm.GetIntValues(NAMESPACE, PROPERTY);
        }

        /// <summary>
        /// GetDoubleValues() with null namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetDoubleValues_NullNamespace()
        {
            cm.GetDoubleValues(null, PROPERTY);
        }

        /// <summary>
        /// GetDoubleValues() with null property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void GetDoubleValues_NullProperty()
        {
            cm.GetDoubleValues(NAMESPACE, null);
        }

        /// <summary>
        /// GetDoubleValues() with empty namespace.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetDoubleValues_EmptyNamespace()
        {
            cm.GetDoubleValues(string.Empty, PROPERTY);
        }

        /// <summary>
        /// GetDoubleValues() with empty property.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void GetDoubleValues_EmptyProperty()
        {
            cm.GetDoubleValues(NAMESPACE, string.Empty);
        }

        /// <summary>
        /// GetDoubleValues() with invalid format.
        /// </summary>
        [Test, ExpectedException(typeof(FormatException))]
        public void GetDoubleValues_InvalidFormat()
        {
            cm.SetValue(NAMESPACE, PROPERTY, "twelve");
            cm.GetDoubleValues(NAMESPACE, PROPERTY);
        }

    }

}
