// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER

using System;
using System.Threading;
using NUnit.Framework;

namespace TopCoder.Util.ConfigurationManager.StressTests {
    /// <summary>
    /// Benchmark tests of new features of ConfigManager version 1.1 .
    /// </summary>
    [TestFixture]
    public class NewBencmarkTests {
        /// <summary>
        /// The prefix for all filenames.
        /// </summary>
        private const string PREFIX = @"..\..\test_files\stress\";

        /// <summary>
        /// how many time to repeat test
        /// </summary>
        private int cnt;

        /// <summary>
        /// instance of StressTestsUtils used for benchmark testing
        /// </summary>
        private StressTestsUtils tu = null;

        /// <summary>
        /// The configuration manager instance.
        /// </summary>
        private ConfigManager cm = null;

        /// <summary>
        /// saved exception to throw in Test. methods
        /// </summary>
        private Exception exc = null;
        
        /// <summary>
        /// Set up.
        /// </summary>
        [SetUp]
        public void SetUp() {
            cm = ConfigManager.GetInstance();
            tu = new StressTestsUtils();
            tu.units = " config per sec";
        }

        /// <summary>
        /// run simple LoadFile for short file
        /// </summary>
        private void RunSimple() {
            try {
                for (int i=0; i<cnt; i++) {
                    cm.Clear();
                    cm.LoadFile(PREFIX + "stress1.xml");
                }
            } catch (Exception e) {
                exc =  e;
            }
        }

        /// <summary>
        /// benchmarking RunSimple
        /// </summary>
        [Test]
        public void TestSimpleWork() {
            ThreadStart tr = new ThreadStart(RunSimple);
            tu.RunInThreads("ConfigManager", "simple", 1000, 1, tr, out cnt); 
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// run LoadFile on Bif ConfigManager file
        /// </summary>
        private void RunBig() {
            try {
                for (int i=0; i<cnt; i++) {
                    cm.Clear();
                    cm.LoadFile(PREFIX + "stressBig.xml");
                }
            } catch (Exception e) {
                exc =  e;
            }
        }

        /// <summary>
        /// benchmarking RunBig
        /// </summary>
        [Test]
        public void TestBigWork() {
            ThreadStart tr = new ThreadStart(RunBig);
            tu.RunInThreads("ConfigManager", "simple", 100, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running CreateInstance of NewClass
        /// </summary>
        private void RunCreate() {
            NewClass newClass = null;
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressCreate.xml");
                BaseClass bc = new BaseClass();
                Type tbase = bc.GetType();

                for (int i = 0; i< cnt; i++) {
                    newClass = (NewClass)cm.CreateInstance("nsCreate", "classname", tbase);
                }
                Assert.AreEqual("NewClass", newClass.name);
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// bechmarking RunCreate
        /// </summary>
        [Test]
        public void TestCreate() {
            ThreadStart tr = new ThreadStart(RunCreate);
            tu.RunInThreads("ConfigManager", "simple", 1000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running create instnce of NewCalss using constructor with arguments.
        /// </summary>
        private void RunCreateArgs() {
            NewClass newClass = null;
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressCreate.xml");
                BaseClass bc = new BaseClass();
                Type tbase = bc.GetType();
                object []args = new object[]{"argument"};

                for (int i = 0; i< cnt; i++) {
                    newClass = (NewClass)cm.CreateInstance("nsCreate", "classname", args, tbase);
                }
                Assert.AreEqual("argument", newClass.name);
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// benchmarking RunCreateArgs
        /// </summary>
        [Test]
        public void TestCreateArgs() {
            ThreadStart tr = new ThreadStart(RunCreateArgs);
            tu.RunInThreads("ConfigManager", "crete args", 1000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// Running Getvalue with string value
        /// </summary>
        private void RunGetValueStr() {
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressValue.xml");
                string val = null;
                for (int i = 0; i< cnt; i++) {
                    val = cm.GetValue("nsValue", "StrValue");
                }
                Assert.AreEqual("string", val, "String Value");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// benchmarking RunGetValueStr
        /// </summary>
        [Test]
        public void TestGetValueStr() {
            ThreadStart tr = new ThreadStart(RunGetValueStr);
            tu.RunInThreads("ConfigManager", "get value str", 10000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// Running GetIntegere value bay calling int.Parse on string value
        /// </summary>
        private void RunGetValueIntA() {
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressValue.xml");
                int val = 0;
                for (int i = 0; i< cnt; i++) {
                    val = int.Parse(cm.GetValue("nsValue", "IntValue"));
                }
                Assert.AreEqual(7654321, val, "Int Value A");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// geting integer value by directly calling GetIntValue
        /// </summary>
        private void RunGetValueIntB() {
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressValue.xml");
                int val = 0;
                for (int i = 0; i< cnt; i++) {
                    val = cm.GetIntValue("nsValue", "IntValue");
                }
                Assert.AreEqual(7654321, val, "Int Value B");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// benchmarking RunGetValueIntA
        /// </summary>
        [Test]
        public void TestGetValueIntA() {
            ThreadStart tr = new ThreadStart(RunGetValueIntA);
            tu.RunInThreads("ConfigManager", "get int value A", 10000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }
        /// <summary>
        /// benchmarking RunGetValueIntB
        /// </summary>
        [Test]
        public void TestGetValueIntB() {
            ThreadStart tr = new ThreadStart(RunGetValueIntB);
            tu.RunInThreads("ConfigManager", "get int vlaue B", 10000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running getting double property by using double.Parse()
        /// </summary>
        private void RunGetValueDoubleA() {
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressValue.xml");
                double val = 0;
                for (int i = 0; i< cnt; i++) {
                    val = double.Parse(cm.GetValue("nsValue", "DoubleValue"));
                }
                Assert.AreEqual(234.988776, val, "Int Value A");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// running get double property by directly calling GetDoubleValue()
        /// </summary>
        private void RunGetValueDoubleB() {
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressValue.xml");
                double val = 0;
                for (int i = 0; i< cnt; i++) {
                    val = cm.GetDoubleValue("nsValue", "DoubleValue");
                }
                Assert.AreEqual(234.988776, val, "Int Value B");
            } catch (Exception e) {
                exc = e;
            }
        }
        /// <summary>
        /// benchmarking RunGetValueDoubleA
        /// </summary>
        [Test]
        public void TestGetValueDoubleA() {
            ThreadStart tr = new ThreadStart(RunGetValueDoubleA);
            tu.RunInThreads("ConfiugManager", "get int value A", 10000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }
        /// <summary>
        /// benchmarking RunGetValueDoubleB
        /// </summary>
        [Test]
        public void TestGetValueDoubleB() {
            ThreadStart tr = new ThreadStart(RunGetValueDoubleB);
            tu.RunInThreads("ConfigManager", "get int vlaue B", 10000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running getting integer array
        /// </summary>
        private void RunReadIntegers() {
            try {
                cm.Clear();
                cm.LoadFile(PREFIX + "stressValue.xml");
                int []val = null;
                for (int i = 0; i< cnt; i++) {
                    val = cm.GetIntValues("nsValue", "IntValues");
                }
                Assert.AreEqual(7654321, val[0], "Int Value [0] B");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// benchmarking RunReadIntegers()
        /// </summary>
        [Test]
        public void TestReadIntegers() {
            ThreadStart tr = new ThreadStart(RunReadIntegers);
            tu.RunInThreads("ConfigManager", "get read integers", 10000, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running saving value of property
        /// </summary>
        private void RunSaveValue() {
            try {
                for (int i = 0; i< cnt; i++) {
                    cm.Clear();
                    cm.LoadFile(PREFIX + "stressValue.xml");
                    cm.SetValue("nsValue", "aaa", "bbbb");
                }
                string val = cm.GetValue("nsValue", "aaa");
                Assert.AreEqual("bbbb", val, "string value");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// benchmarking RunSaveValue()
        /// </summary>
        [Test]
        public void TestSaveValue() {
            ThreadStart tr = new ThreadStart(RunSaveValue);
            tu.RunInThreads("ConfigManager", "save value", 100, 1, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }
    }
}
